@@ -1,10 +1,10 @@
-;;; ob-comint.el --- org-babel functions for interaction with comint buffers
+;;; ob-comint.el --- Babel Functions for Interaction with Comint Buffers -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2009-2015 Free Software Foundation, Inc.
+;; Copyright (C) 2009-2019 Free Software Foundation, Inc.
 
 ;; Author: Eric Schulte
 ;; Keywords: literate programming, reproducible research, comint
-;; Homepage: http://orgmode.org
+;; Homepage: https://orgmode.org
 
 ;; This file is part of GNU Emacs.
 
@@ -19,7 +19,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -33,9 +33,6 @@
 (require 'ob-core)
 (require 'org-compat)
 (require 'comint)
-(eval-when-compile (require 'cl))
-(declare-function with-parsed-tramp-file-name "tramp" (filename var &rest body))
-(declare-function tramp-flush-directory-property "tramp" (vec directory))
 
 (defun org-babel-comint-buffer-livep (buffer)
   "Check if BUFFER is a comint buffer with a live process."
@@ -48,12 +45,14 @@ BUFFER is checked with `org-babel-comint-buffer-livep'.  BODY is
 executed inside the protection of `save-excursion' and
 `save-match-data'."
   (declare (indent 1))
-  `(save-excursion
+  `(progn
+     (unless (org-babel-comint-buffer-livep ,buffer)
+       (error "Buffer %s does not exist or has no process" ,buffer))
      (save-match-data
-       (unless (org-babel-comint-buffer-livep ,buffer)
-         (error "Buffer %s does not exist or has no process" ,buffer))
-       (set-buffer ,buffer)
-       ,@body)))
+       (with-current-buffer ,buffer
+	 (save-excursion
+	   (let ((comint-input-filter (lambda (_input) nil)))
+	     ,@body))))))
 (def-edebug-spec org-babel-comint-in-buffer (form body))
 
 (defmacro org-babel-comint-with-output (meta &rest body)
@@ -69,53 +68,49 @@ elements are optional.
 This macro ensures that the filter is removed in case of an error
 or user `keyboard-quit' during execution of body."
   (declare (indent 1))
-  (let ((buffer (car meta))
-	(eoe-indicator (cadr meta))
-	(remove-echo (cadr (cdr meta)))
-	(full-body (cadr (cdr (cdr meta)))))
+  (let ((buffer (nth 0 meta))
+	(eoe-indicator (nth 1 meta))
+	(remove-echo (nth 2 meta))
+	(full-body (nth 3 meta)))
     `(org-babel-comint-in-buffer ,buffer
-       (let ((string-buffer "") dangling-text raw)
-	 ;; setup filter
-	 (setq comint-output-filter-functions
+       (let* ((string-buffer "")
+	      (comint-output-filter-functions
 	       (cons (lambda (text) (setq string-buffer (concat string-buffer text)))
 		     comint-output-filter-functions))
-	 (unwind-protect
-	     (progn
-	       ;; got located, and save dangling text
-	       (goto-char (process-mark (get-buffer-process (current-buffer))))
-	       (let ((start (point))
-		     (end (point-max)))
-		 (setq dangling-text (buffer-substring start end))
-		 (delete-region start end))
-	       ;; pass FULL-BODY to process
-	       ,@body
-	       ;; wait for end-of-evaluation indicator
-	       (while (progn
-			(goto-char comint-last-input-end)
-			(not (save-excursion
-			       (and (re-search-forward
-				     (regexp-quote ,eoe-indicator) nil t)
-				    (re-search-forward
-				     comint-prompt-regexp nil t)))))
-		 (accept-process-output (get-buffer-process (current-buffer)))
-		 ;; thought the following this would allow async
-		 ;; background running, but I was wrong...
-		 ;; (run-with-timer .5 .5 'accept-process-output
-		 ;; 		 (get-buffer-process (current-buffer)))
-		 )
-	       ;; replace cut dangling text
-	       (goto-char (process-mark (get-buffer-process (current-buffer))))
-	       (insert dangling-text))
-	   ;; remove filter
-	   (setq comint-output-filter-functions
-		 (cdr comint-output-filter-functions)))
+	      dangling-text)
+	 ;; got located, and save dangling text
+	 (goto-char (process-mark (get-buffer-process (current-buffer))))
+	 (let ((start (point))
+	       (end (point-max)))
+	   (setq dangling-text (buffer-substring start end))
+	   (delete-region start end))
+	 ;; pass FULL-BODY to process
+	 ,@body
+	 ;; wait for end-of-evaluation indicator
+	 (while (progn
+		  (goto-char comint-last-input-end)
+		  (not (save-excursion
+			 (and (re-search-forward
+			       (regexp-quote ,eoe-indicator) nil t)
+			      (re-search-forward
+			       comint-prompt-regexp nil t)))))
+	   (accept-process-output (get-buffer-process (current-buffer)))
+	   ;; thought the following this would allow async
+	   ;; background running, but I was wrong...
+	   ;; (run-with-timer .5 .5 'accept-process-output
+	   ;; 		 (get-buffer-process (current-buffer)))
+	   )
+	 ;; replace cut dangling text
+	 (goto-char (process-mark (get-buffer-process (current-buffer))))
+	 (insert dangling-text)
+
 	 ;; remove echo'd FULL-BODY from input
-	 (if (and ,remove-echo ,full-body
-		  (string-match
-		   (replace-regexp-in-string
-		    "\n" "[\r\n]+" (regexp-quote (or ,full-body "")))
-		   string-buffer))
-	     (setq raw (substring string-buffer (match-end 0))))
+	 (when (and ,remove-echo ,full-body
+		    (string-match
+		     (replace-regexp-in-string
+		      "\n" "[\r\n]+" (regexp-quote (or ,full-body "")))
+		     string-buffer))
+	   (setq string-buffer (substring string-buffer (match-end 0))))
 	 (split-string string-buffer comint-prompt-regexp)))))
 (def-edebug-spec org-babel-comint-with-output (sexp body))
 
@@ -148,15 +143,10 @@ Don't return until FILE exists.  Code in STRING must ensure that
 FILE exists at end of evaluation."
   (unless (org-babel-comint-buffer-livep buffer)
     (error "Buffer %s does not exist or has no process" buffer))
-  (if (file-exists-p file) (delete-file file))
+  (when (file-exists-p file) (delete-file file))
   (process-send-string
    (get-buffer-process buffer)
-   (if (string-match "\n$" string) string (concat string "\n")))
-  ;; From Tramp 2.1.19 the following cache flush is not necessary
-  (if (file-remote-p default-directory)
-      (let (v)
-	(with-parsed-tramp-file-name default-directory nil
-				     (tramp-flush-directory-property v ""))))
+   (if (= (aref string (1- (length string))) ?\n) string (concat string "\n")))
   (while (not (file-exists-p file)) (sit-for (or period 0.25))))
 
 (provide 'ob-comint)
