@@ -3,12 +3,12 @@
 #include "copyright.h"
 
 /*
-Copyright (C) 2001-2014 Free Software Foundation, Inc.
+Copyright (C) 2001-2019 Free Software Foundation, Inc.
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+the Free Software Foundation, either version 3 of the License, or (at
+your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,7 +16,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 /*
  * XMenu:	MIT Project Athena, X Window system menu package
@@ -571,6 +571,7 @@ XMenuActivate(
 						   event.xbutton.window
 						   );
 		if (event_xmp != NULL) continue;
+		FALLTHROUGH;
 	    default:
 		/*
 		 * This is a foreign event.
